@@ -9,9 +9,8 @@ from trytond.wizard import Wizard, StateTransition, StateView, Button
 from trytond.pyson import Eval, Bool, If, Id
 from trytond.pool import Pool
 from trytond.transaction import Transaction
-from trytond import backend
 
-from trytond.modules.product import price_digits, TemplateFunction
+from trytond.modules.product import price_digits
 
 __all__ = ['Production', 'AssignFailed', 'Assign']
 
@@ -178,6 +177,7 @@ class Production(Workflow, ModelSQL, ModelView):
                 'cancel': {
                     'invisible': ~Eval('state').in_(['request', 'draft',
                             'assigned']),
+                    'depends': ['state'],
                     },
                 'draft': {
                     'invisible': ~Eval('state').in_(['request', 'waiting',
@@ -185,30 +185,36 @@ class Production(Workflow, ModelSQL, ModelView):
                     'icon': If(Eval('state') == 'cancel',
                         'tryton-clear',
                         If(Eval('state') == 'request',
-                            'tryton-go-next',
-                            'tryton-go-previous')),
+                            'tryton-forward',
+                            'tryton-back')),
+                    'depends': ['state'],
                     },
                 'reset_bom': {
                     'invisible': (~Eval('bom')
                         | ~Eval('state').in_(['request', 'draft', 'waiting'])),
+                    'depends': ['state', 'bom'],
                     },
                 'wait': {
                     'invisible': ~Eval('state').in_(['draft', 'assigned',
                             'waiting', 'running']),
                     'icon': If(Eval('state').in_(['assigned', 'running']),
-                        'tryton-go-previous',
+                        'tryton-back',
                         If(Eval('state') == 'waiting',
                             'tryton-clear',
-                            'tryton-go-next')),
+                            'tryton-forward')),
+                    'depends': ['state'],
                     },
                 'run': {
                     'invisible': Eval('state') != 'assigned',
+                    'depends': ['state'],
                     },
                 'done': {
                     'invisible': Eval('state') != 'running',
+                    'depends': ['state'],
                     },
                 'assign_wizard': {
                     'invisible': Eval('state') != 'waiting',
+                    'depends': ['state'],
                     },
                 'assign_try': {},
                 'assign_force': {},
@@ -216,8 +222,7 @@ class Production(Workflow, ModelSQL, ModelView):
 
     @classmethod
     def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
-        table_h = TableHandler(cls, module_name)
+        table_h = cls.__table_handler__(module_name)
         table = cls.__table__()
 
         # Migration from 3.8: rename code into number
@@ -298,10 +303,8 @@ class Production(Workflow, ModelSQL, ModelView):
     def explode_bom(self):
         pool = Pool()
         Uom = pool.get('product.uom')
-        Move = pool.get('stock.move')
         if not (self.bom and self.product and self.uom):
             return
-        self.cost = Decimal(0)
 
         if self.warehouse:
             storage_location = self.warehouse.storage_location
@@ -319,14 +322,8 @@ class Production(Workflow, ModelSQL, ModelView):
                 inputs.append(move)
                 quantity = Uom.compute_qty(input_.uom, quantity,
                     input_.product.default_uom, round=False)
-                self.cost += (Decimal(str(quantity)) *
-                    input_.product.cost_price)
         self.inputs = inputs
-        digits = self.__class__.cost.digits
-        self.cost = self.cost.quantize(Decimal(str(10 ** -digits[1])))
 
-        digits = Move.unit_price.digits
-        digit = Decimal(str(10 ** -digits[1]))
         outputs = []
         for output in self.bom.outputs:
             quantity = output.compute_quantity(factor)
@@ -334,9 +331,6 @@ class Production(Workflow, ModelSQL, ModelView):
                 self.company, output, quantity)
             if move:
                 move.unit_price = Decimal(0)
-                if output.product == move.product and quantity:
-                    move.unit_price = Decimal(
-                        self.cost / Decimal(str(quantity))).quantize(digit)
                 outputs.append(move)
         self.outputs = outputs
 
@@ -418,9 +412,6 @@ class Production(Workflow, ModelSQL, ModelView):
         return cost
 
     def set_moves(self):
-        pool = Pool()
-        Move = pool.get('stock.move')
-
         storage_location = self.warehouse.storage_location
         location = self.location
         company = self.company
@@ -437,7 +428,6 @@ class Production(Workflow, ModelSQL, ModelView):
             return
 
         factor = self.bom.compute_factor(self.product, self.quantity, self.uom)
-        cost = Decimal(0)
         for input_ in self.bom.inputs:
             quantity = input_.compute_quantity(factor)
             product = input_.product
@@ -446,13 +436,7 @@ class Production(Workflow, ModelSQL, ModelView):
             if move:
                 move.production_input = self
                 move.save()
-                cost += (Decimal(str(move.internal_quantity)) *
-                    product.cost_price)
-        digits = self.__class__.cost.digits
-        cost = cost.quantize(Decimal(str(10 ** -digits[1])))
 
-        digits = Move.unit_price.digits
-        digit = Decimal(str(10 ** -digits[1]))
         for output in self.bom.outputs:
             quantity = output.compute_quantity(factor)
             product = output.product
@@ -460,27 +444,50 @@ class Production(Workflow, ModelSQL, ModelView):
                 product, output.uom, quantity)
             if move:
                 move.production_output = self
-                if product == self.product:
-                    move.unit_price = Decimal(
-                        cost / Decimal(str(quantity))).quantize(digit)
-                else:
-                    move.unit_price = Decimal(0)
+                move.unit_price = Decimal(0)
                 move.save()
         self._set_move_planned_date()
 
+    @classmethod
+    def set_cost(cls, productions):
+        pool = Pool()
+        Uom = pool.get('product.uom')
+        Move = pool.get('stock.move')
+
+        digits = Move.unit_price.digits
+        digit = Decimal(str(10 ** -digits[1]))
+        moves = []
+        for production in productions:
+            if not production.quantity or not production.uom:
+                continue
+            if production.company.currency.is_zero(
+                    production.cost - production.output_cost):
+                continue
+            unit_price = production.cost / Decimal(str(production.quantity))
+            for output in production.outputs:
+                if output.product == production.product:
+                    output.unit_price = Uom.compute_price(
+                        production.uom, unit_price, output.uom).quantize(digit)
+                    moves.append(output)
+        Move.save(moves)
+
     @classmethod
     def validate(cls, productions):
         super(Production, cls).validate(productions)
         for production in productions:
             production.check_cost()
 
+    @property
+    def output_cost(self):
+        cost_price = Decimal(0)
+        for output in self.outputs:
+            cost_price += (Decimal(str(output.quantity)) * output.unit_price)
+        return cost_price
+
     def check_cost(self):
         if self.state != 'done':
             return
-        cost_price = Decimal(0)
-        for output in self.outputs:
-            cost_price += (Decimal(str(output.quantity))
-                * output.unit_price)
+        cost_price = self.output_cost
         if not self.company.currency.is_zero(self.cost - cost_price):
             self.raise_user_error('uneven_costs', {
                     'production': self.rec_name,
@@ -514,7 +521,8 @@ class Production(Workflow, ModelSQL, ModelView):
     def copy(cls, productions, default=None):
         if default is None:
             default = {}
-        default = default.copy()
+        else:
+            default = default.copy()
         default.setdefault('number', None)
         return super(Production, cls).copy(productions, default=default)
 
@@ -588,6 +596,7 @@ class Production(Workflow, ModelSQL, ModelView):
         pool = Pool()
         Move = pool.get('stock.move')
         Date = pool.get('ir.date')
+        cls.set_cost(productions)
         Move.do([m for p in productions for m in p.outputs])
         cls.write([p for p in productions if not p.effective_date], {
                 'effective_date': Date.today(),
@@ -595,7 +604,7 @@ class Production(Workflow, ModelSQL, ModelView):
 
     @classmethod
     @ModelView.button_action('production.wizard_assign')
-    def assign_wizard(self, productions):
+    def assign_wizard(cls, productions):
         pass
 
     @classmethod
@@ -643,7 +652,7 @@ class Assign(Wizard):
     start = StateTransition()
     failed = StateView('production.assign.failed',
         'production.assign_failed_view_form', [
-            Button('Force Assign', 'force', 'tryton-go-next',
+            Button('Force Assign', 'force', 'tryton-forward',
                 states={
                     'invisible': ~Id('stock',
                         'group_stock_force_assignment').in_(
